#include "recovery.hh"
#include "../gui/recovery_report.hh"
#include <sys/time.h>

extern struct ncfs_state* NCFS_DATA;
extern FileSystemLayer* fileSystemLayer;
extern CacheLayer* cacheLayer;

void* ProcessThread(void* arg);
int Recover(int fail_disk_id);
pthread_t _ThreadID;
char _newdevice[PATH_MAX];

void RecoveryTool::Recover(char* newdevice){

	//_newdevice = newdevice;
	memset(_newdevice,0,PATH_MAX);
	strncpy(_newdevice,newdevice,strlen(newdevice));
	pthread_create(&_ThreadID,NULL,ProcessThread,NULL);
	//ProcessThread(NULL);
	//fprintf(stderr,"\n\n\nCreating thread for recovery, TID %lu device %s\n\n\n",(unsigned long)_ThreadID,newdevice);
}

int Recover(int fail_disk_id){
	NCFS_DATA->process_state = 1;
	RecoveryReport::Start(_newdevice);
	//int __recoversize = NCFS_DATA->free_offset[fail_disk_id];
	int __recoversize = NCFS_DATA->disk_size[fail_disk_id];

        struct timeval starttime, endtime;
        struct timeval t1, t2;
	double duration;
	double data_size;

	gettimeofday(&starttime,NULL);
	NCFS_DATA->dev_name[fail_disk_id] = (char*)realloc(NCFS_DATA->dev_name[fail_disk_id],strlen(_newdevice));
	memset(NCFS_DATA->dev_name[fail_disk_id],0,strlen(_newdevice));
	strncpy(NCFS_DATA->dev_name[fail_disk_id],_newdevice,strlen(_newdevice));
	cacheLayer->DiskCacheName(fail_disk_id,_newdevice);
	for(int i = 0; i < __recoversize; ++i){
		int block_size = NCFS_DATA->disk_block_size;
		int offset = i * block_size;
		//(bug@@)char *buf = (char*)calloc(block_size,0);
		char buf[block_size];
		int retstat = fileSystemLayer->codingLayer->decode(fail_disk_id,buf,block_size,offset);

		if (NCFS_DATA->run_experiment == 1){
			gettimeofday(&t1,NULL);
		}

		retstat = cacheLayer->DiskWrite(fail_disk_id,buf,block_size,offset);

                if (NCFS_DATA->run_experiment == 1){
                        gettimeofday(&t2,NULL);

        		duration = t2.tv_sec - t1.tv_sec + 
				(t2.tv_usec - t1.tv_usec)/1000000.0;
			NCFS_DATA->diskwrite_time += duration;
                }

		//(bug@@)free(buf);
	}
	//NCFS_DATA->disk_status[fail_disk_id] = 0;
	fileSystemLayer->set_device_status(fail_disk_id,0);

	gettimeofday(&endtime,NULL);

	fprintf(stderr,"\n\n\nRecovery on disk %d, new device %s Done.\n\n\n",fail_disk_id,_newdevice);

        duration = endtime.tv_sec - starttime.tv_sec + (endtime.tv_usec-starttime.tv_usec)/1000000.0;
	data_size  = __recoversize * (NCFS_DATA->disk_block_size) / (1024 * 1024);
        printf("Elapsed time = %fs\n",duration);
        printf("Throughput = %f MB/s\n",(float)(data_size / duration));
	printf("Data size = %f MB\n",(float)data_size);

	RecoveryReport::Finish();
	NCFS_DATA->process_state = 0;
	return 0;
}

void* ProcessThread(void* arg){
	int coding_type = NCFS_DATA->disk_raid_type;
	int disk_total_num = NCFS_DATA->disk_total_num;
	int fail_disk_num = 0;
	int fail_disk_id = 0;
	for(int i = disk_total_num-1; i >= 0; --i){
		if(NCFS_DATA->disk_status[i] !=0){
			++fail_disk_num;
			fail_disk_id = i;
		}
	}

	if (fail_disk_num > 0){
		if (fail_disk_num == 1){
			Recover(fail_disk_id);
		}
		else if ((fail_disk_num == 2) && (coding_type == 6)){
			Recover(fail_disk_id);
		}
		else if ((coding_type == 1000) &&
				(fail_disk_num <= NCFS_DATA->mbr_n - NCFS_DATA->mbr_k)){
			Recover(fail_disk_id);
		}
                else if ((coding_type == 2000) &&
                                (fail_disk_num <= NCFS_DATA->msr_n - NCFS_DATA->msr_k)){
                        Recover(fail_disk_id);
                }
		else{
			printf("Too many disks fail.\n");
		}
	}
	else{
		printf("No disk fails.\n");
	}

	return NULL;
}
